'''implements a wirelength estimation algorithm

@author: Robert Fischbach
@author: Tobias Heimpold
@copyright: Copyright (C) 2011
@license: This program is free software; you can redistribute it and/or modify it 
          under the terms of the GNU General Public License as published by the 
          Free Software Foundation; either version 3 of the License, or (at your 
          option) any later version.
          This program is distributed in the hope that it will be useful, but 
          WITHOUT ANY WARRANTY; without even the implied warranty of 
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
          Public License for more details.
          You should have received a copy of the GNU General Public License along 
          with this program; if not, see <http://www.gnu.org/licenses/>.
@contact: For questions and bugs send an email to <fischbach (at) ifte.de>
'''

def middle_point_manhatten(a, b, z_weight=1.0):
    """Calculated the manhatten distance between two points either in 2D or 3D.
    Information is either [x, y, w, h] or [x, y, z, w, h, d].
    
    @param a: Information about point A.
    @type a: list-of-float
    
    @param b: Information about point B.
    @type b: list-of-float
    """
    from exceptions import ValueError
    
    dims = len(a)/2
    assert(len(a) == len(b))

    length = 0.0
    if dims == 2: # 2D packing
        length += abs((a[0]+a[2]/2) - (b[0]+b[2]/2)) # x
        length += abs((a[1]+a[3]/2) - (b[1]+b[3]/2)) # y
    elif dims == 3: # 3D packing
        length += abs((a[0]+a[3]/2) - (b[0]+b[3]/2)) # x
        length += abs((a[1]+a[4]/2) - (b[1]+b[4]/2)) # y
        length += z_weight*abs((a[2]+a[5]/2) - (b[2]+b[5]/2)) # z
    else:
        raise ValueError("wrong packing format")
        
    return length

