"""implements several geometrical operations

@author: Robert Fischbach
@author: Tobias Heimpold
@copyright: Copyright (C) 2011
@license: This program is free software; you can redistribute it and/or modify it 
          under the terms of the GNU General Public License as published by the 
          Free Software Foundation; either version 3 of the License, or (at your 
          option) any later version.
          This program is distributed in the hope that it will be useful, but 
          WITHOUT ANY WARRANTY; without even the implied warranty of 
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
          Public License for more details.
          You should have received a copy of the GNU General Public License along 
          with this program; if not, see <http://www.gnu.org/licenses/>.
@contact: For questions and bugs send an email to <fischbach (at) ifte.de>
"""

# I still have to put it in order (classes or at least functional groups)

# helper function testing overlaping of two finite linesegments
def overlap1d(a_left, a_right, b_left, b_right):
    """Tests for an overlap in 1D
    Tests two lines a and b for overlap in 1D.
    
    @param a_left: Left, lower edge of line a.
    @type a_left: float
    
    @param a_right: Right, upper edge of line a.
    @type a_right: float
    
    @param b_left: Left, lower edge of line b.
    @type b_left: float
    
    @param b_right: Right, upper edge of line b.
    @type b_right: float
    
    @return: Returns the test result as boolean.
        - True: overlap
        - False: no overlap
    @rtype: boolean
    """
    if (a_right <= b_left) or (b_right <= a_left):
        return False
    else:
        return True

def overlap2d(a11, a12, a21, a22, b11, b12, b21, b22):
    """Tests for an overlap in 2D
    Tests two boxes a and b for overlap in 1D.
    
    @param a11: Left, lower edge of box a.
    @type a11: float
    
    @param a12: Right, lower edge of box a.
    @type a12: float
    
    @param a21: Left, upper edge of box a.
    @type a21: float
    
    @param a22: Right, upper edge of box a.
    @type a22: float
    
    @param b11: Left, lower edge of box b.
    @type b11: float
    
    @param b12: Right, lower edge of box b.
    @type b12: float
    
    @param b21: Left, upper edge of box b.
    @type b21: float
    
    @param b22: Right, upper edge of box b.
    @type b22: float
    
    @return: Returns the test result as boolean.
        - True: overlap
        - False: no overlap
    @rtype: boolean
    """
    if (a12 <= b11) or (b12 <= a11) or (a22 <= b21) or (b22 <= a21):
        return False
    else:
        return True


def valid_packing(packing):
    """Returns True, Does nothing"""
    return True
