"""implements several graph theory structures and algorithms

@author: Robert Fischbach
@author: Tobias Heimpold
@copyright: Copyright (C) 2011
@license: This program is free software; you can redistribute it and/or modify it 
          under the terms of the GNU General Public License as published by the 
          Free Software Foundation; either version 3 of the License, or (at your 
          option) any later version.
          This program is distributed in the hope that it will be useful, but 
          WITHOUT ANY WARRANTY; without even the implied warranty of 
          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
          Public License for more details.
          You should have received a copy of the GNU General Public License along 
          with this program; if not, see <http://www.gnu.org/licenses/>.
@contact: For questions and bugs send an email to <fischbach (at) ifte.de>
"""

from Queue import Queue
from dsc_suite.tools.math import catalan_number
from scipy.misc import comb
from random import randint, randrange

# graph expecting source (target are all nodes without children)
def longest_path(graph, weights):
    """Not documented yet.
        
        @param graph:
        @type graph:
        
        @param weights:
        @type weights:
        
        @return:
        @rtype: dictionary, dictionary
        
        @todo: Create docstring.
        """
    # wird auch in slicing-tree verwendet, bei aenderungen angleichen
    # noch uneffizient, da mehrfachens untersuchen der knoten
    # moeglich. ist zur zeit aber noch notwendig, da es redundante
    # kanten gibt.
    # besser vorher red. kanten entfernen
    # algorithmuss gibt graph ohne transitive kanten zurueck (parent)
    distance = dict()
    parent = dict()
    for b in graph.keys():
        distance[b] = 0
        parent[b] = ''
    q = Queue()
    q.put('Source')
    while not q.empty():
        u = q.get()
        for v in graph[u]:
            if distance[u]+weights[v] > distance[v]:
                distance[v] = distance[u]+weights[v]
                parent[v] = u
                q.put(v)
    return distance, parent

##### TERNARY TREES ###############################################
class TernaryTrees(object):
    """Implements basic TernaryTree functionality.
    This class implements basic ternary functionality, such as
    ternary tree numbering and generation.
    """
    LEFT = 0
    MIDDLE = 1
    RIGHT = 2
    PARENT = 3
    INTERNAL = 4
    DEPTH = 5
    
    @staticmethod
    def get_num_ternary_trees(n):
        """Calculates number of possible ternary trees.
        
        Binomial(3n,n)/(2n+1)
        (enumerates ternary trees and also non-crossing trees) 
        http://www.research.att.com/~njas/sequences/A001764
        
        @param n: number of nodes
        @type n: int
        """
        return int(round(comb(3*n, n) / (2*n + 1)))

    @staticmethod
    def get_random_ternary_tree(N):
        """Generates a random ternary tree. Evenly distributed.
        Generates an evenly distributed random ternary tree.
        Approach taken from [Panholzer2002]. Returned format
        is a complete ternary tree with internal and external
        nodes. To get a random ternary tree only the internal
        nodes are to use.
        
        @note: Depth information was intended to speed up the descendant
        test. The depth assignment is erroneous. Some parent-child nodes
        have the same depth. -> Don't use until resolved. 
                
        @param N: number of (internal) nodes
        @type N: int
        
        @return: Returns ternary tree in a list format where each entry
        represents a node. The index of the root node is also returned.        
        T = [[LEFT, MIDDLE, RIGHT, PARENT, INTERNAL, DEPTH], ...]
        @rtype: list-of-, list-of-
        """
        n = 0
        root = 0
        T = [[None, None, None, None, False, 0]]
        while not n == N:
            n += 1
            # pick node randomly, with color
            x = randrange(len(T))
            color = randrange(3)
            y = randrange(len(T)+1) # if y == len(T) -> x-tra node
            if y == len(T):
                y = 'extra node' 
            # indices of new nodes
            p = len(T)
            a = len(T) + 1
            b = len(T) + 2
            # update T with new p, x and a
            T.append([None, None, None, T[x][TernaryTrees.PARENT], True, T[x][TernaryTrees.DEPTH]])
            # insert a depending on chosen color
            T[p][color] = a 
            # insert old x as cyclic right brother of a
            T[p][(color+1)%3] = x
            # modify x node
            if not T[x][TernaryTrees.PARENT] == None:
                T[T[x][TernaryTrees.PARENT]][T[T[x][TernaryTrees.PARENT]].index(x)] = p
            else:
                root = p
            T[x][TernaryTrees.PARENT] = p 
            T[x][TernaryTrees.DEPTH] = T[p][TernaryTrees.DEPTH] + 1
            # append node a  
            T.append([None, None, None, p, False, T[p][TernaryTrees.DEPTH]+1])
            # test if p is a descendant of y
            """
            TIEFENZUWEISUNG IST NOCH FEHLERHAFT, ALSO NICHT VERWENDEN BIS
            FEHLER GEFUNDEN WIRD. SYMPTOM: ELTERNKNOTEN HABEN MANCHMAL DIE
            SELBE TIEFE WIE IHRE KINDKNOTEN (BZW. ANDERSRUM).
            p_desc_of_y = False
            if not y == 'extra node' and T[p][DEPTH] > T[y][DEPTH]:
                parent = T[p][PARENT]
                while T[parent][DEPTH] > T[y][DEPTH]: #TODO check depth
                    #print T[parent][DEPTH], T[y][DEPTH]
                    if T[T[parent][PARENT]][DEPTH] >= T[parent][DEPTH]:
                        print p, y
                        for i in range(len(T)):
                            print i, T[i]
                        return None
                    parent = T[parent][PARENT]
                if parent == y:
                    p_desc_of_y = True
            """
            p_desc_of_y = False
            parent = T[p][TernaryTrees.PARENT]
            while not parent == None:
                if parent == y:
                    p_desc_of_y = True
                    break
                parent = T[parent][TernaryTrees.PARENT]
            # take care for leaf labeled b
            if not p_desc_of_y:
                # in this case y cannot be root
                if y == 'extra node':
                    # y == b
                    T.append([None, None, None, p, False, T[p][TernaryTrees.DEPTH]+1])
                    # y(b) is cyclic left brother of a
                    T[p][(color-1)%3] = b
                else:
                    # b substitutes y
                    T.append([None, None, None, T[y][TernaryTrees.PARENT], False, T[y][TernaryTrees.DEPTH]])
                    T[T[y][TernaryTrees.PARENT]][T[T[y][TernaryTrees.PARENT]].index(y)] = b
                    # y is cyclic left brother of a
                    T[p][(color-1)%3] = y
                    T[y][TernaryTrees.PARENT] = p
                    T[y][TernaryTrees.DEPTH] = T[p][TernaryTrees.DEPTH]+1
            else:
                # rotate b -> f(a) -> y -> cl(a)
                T.append([None, None, None, T[p][TernaryTrees.PARENT], False, T[p][TernaryTrees.DEPTH]])
                T[T[p][TernaryTrees.PARENT]][T[T[p][TernaryTrees.PARENT]].index(p)] = b
                # p substitutes y
                T[p][TernaryTrees.PARENT] = T[y][TernaryTrees.PARENT]
                T[p][TernaryTrees.DEPTH] = T[y][TernaryTrees.DEPTH]
                if not T[y][TernaryTrees.PARENT] == None:
                    T[T[y][TernaryTrees.PARENT]][T[T[y][TernaryTrees.PARENT]].index(y)] = p
                else:
                    root = p
                # y is cyclic left brother of a
                T[p][(color-1)%3] = y
                T[y][TernaryTrees.PARENT] = p
                T[y][TernaryTrees.DEPTH] = T[p][TernaryTrees.DEPTH]+1
        return T, root

    @staticmethod
    def transform_panholzer_to_topology(T, root):
        """Not documented yet.
        
        @param T:
        @type T:
        
        @param root:
        @type root:
        
        @return:
        @rtype: 
        
        @todo: Create docstring.
        """
        topology = []
        dfs_order = []
        TernaryTrees.__transform_panholzer_to_topology_recursive(T, root,
                                                                 topology,
                                                                 dfs_order)
        for i in range(len(topology)):
            for j in range(4): # parent, left, middle, right
                if topology[i][j] in dfs_order:
                    topology[i][j] = dfs_order.index(topology[i][j])
                else:
                    topology[i][j] = None
        return topology
        
    @staticmethod        
    def __transform_panholzer_to_topology_recursive(T, node, topology, dfs_order, parent=None):
        """Not documented yet.
        
        @param T:
        @type T:
        
        @param node:
        @type node:
        
        @param topology:
        @type topology:
        
        @param dfs_order:
        @type dfs_order:
        
        @param parent:
        @type parent:
        
        @todo: Create docstring.
        """
        if T[node][TernaryTrees.INTERNAL] == True:
            dfs_order.append(node)
            topology.append([parent,
                             T[node][TernaryTrees.LEFT],
                             T[node][TernaryTrees.MIDDLE],
                             T[node][TernaryTrees.RIGHT]])
            TernaryTrees.__transform_panholzer_to_topology_recursive(T,
                                                                     T[node][TernaryTrees.LEFT],
                                                                     topology,
                                                                     dfs_order,
                                                                     node)
            TernaryTrees.__transform_panholzer_to_topology_recursive(T,
                                                                     T[node][TernaryTrees.MIDDLE],
                                                                     topology,
                                                                     dfs_order,
                                                                     node)
            TernaryTrees.__transform_panholzer_to_topology_recursive(T,
                                                                     T[node][TernaryTrees.RIGHT],
                                                                     topology,
                                                                     dfs_order,
                                                                     node)            


##### BINARY TREES ################################################
class BinaryTrees(object):
    """Implements basic BT functionality.
    This class implements basic binary tree functionality, such as
    binary tree generation and numbering.
    Most methods are static (class) methods. Basically, this class
    is used to encapsulated conjoined functions.
    """

    @staticmethod
    def get_num_binary_trees(n):
        """Calculates number of possible binary trees.
        @param n: number of inner nodes
        @type n: int
        
        @return: Returns catalan number of n.
        @rtype: int
        """
        return catalan_number(n)

    @staticmethod
    def get_binary_tree(n, i):
        """Generates number i binary tree as a binary in order sequence.
        From all possible binary trees with n inner nodes the i'tht tree
        is generated as an pre-order binary sequence. Algorithm by Ruskey1978.
        
        @param n: number of inner nodes
        @type n: int
        
        @param i: index of binary tree, 0 <= i < catalan_number(n)
        @type i: int
        
        @return:
        @rtype: string
        """
        N = i + 1
        temp = ''
        q = n
        m = p = c = 1
        while p < n:
            p = p + 1
            c = ((4 * p - 2) * c) / (p + 1)
        while q > 0:
            cc = ((q + 1) * (q - p) * c) / ((q + p) * (q - p + 1))
            if N <= cc:
                q = q - 1
                c = cc
                temp += '0'
                m = m + 1
            else:
                p = p - 1
                c = c - cc
                N = N - cc
                temp += '1'
                m = m + 1
        return temp + '0'

    @staticmethod
    def generate_relations(topology):
        """Generates topology relations from a given pre-order representation.
        
        @param topology:
        @type topology:
        
        @return:
        @rtype: list-of-
        """
        open_right = [0]
        # which inner node misses it's right child
        result = [[None, 1], [0]]
        # result list: [[parent, left child, right child], ...]
        i = 1
        while i < len(topology):
            if topology[i] == '1':
                # inner node
                if len(result) == i + 1:
                    # parent is already in result list
                    result[i].append(i+1)
                    # next node is left child
                else:
                    # obtain parent from open_right list
                    parent = open_right.pop()
                    result[parent].append(i)
                    result.append([parent, i+1])
                # append parent for left node
                result.append([i])
                open_right.append(i)
            else:
                # leaf
                if len(result) == i + 1:
                    # parent is already in result list
                    result[i].append(None)
                    result[i].append(None)
                    # leaves don't have children
                else:
                    # obtain parent from open_right list
                    parent = open_right.pop()
                    result[parent].append(i)
                    result.append([parent, None, None])
            i += 1
        return result

    @staticmethod
    def generate_inversion_table(n, result, i=0, d=0, s=[]):
        """Not documented yet.
        
        @param n:
        @type n: int
        
        @param result:
        @type result:
        
        @param i:
        @type i: int
        
        @param d:
        @type d: int
        
        @param s:
        @type s: list-of-
        
        @todo: Create docstring.
        """
        if d < n:
            for j in range(i+2):
                BinaryTrees.generate_inversion_table(n, result, j, d + 1, list(s)+[j])
        else:
            result.append(s)

    @staticmethod
    def get_binary_trees(n):
        """Not documented yet.
        
        @param n:
        @type n: int
        
        @return:
        @rtype:
        
        @todo: Create docstring.
        """
        result = []
        i = 0
        i_max = BinaryTrees.get_num_binary_trees(n)
        while i < i_max:
            result.append(BinaryTrees.get_binary_tree(n, i))
            i += 1
        return result